@@ -14,6 +14,7 @@ from __future__ import division, print_function, absolute_import
 import codecs
 import copy
 from datetime import timedelta, datetime
+from difflib import get_close_matches
 from importlib import import_module
 import re
 import os
@@ -31,11 +32,12 @@ from .scrapers import _scraper_dict, _reset_dict, _import_matplotlib
 from .docs_resolv import embed_code_links
 from .downloads import generate_zipfiles
 from .sorting import NumberOfCodeLinesSortKey
-from .binder import copy_binder_files
+from .binder import copy_binder_files, check_binder_conf
 from .directives import MiniGallery
 
 
-_KNOWN_CSS = ('gallery', 'gallery-binder', 'gallery-dataframe')
+_KNOWN_CSS = ('gallery', 'gallery-binder', 'gallery-dataframe',
+              'gallery-rendered-html')
 
 
 class DefaultResetArgv:
@@ -67,9 +69,11 @@ DEFAULT_GALLERY_CONF = {
     'plot_gallery': 'True',
     'download_all_examples': True,
     'abort_on_example_error': False,
+    'only_warn_on_example_error': False,
     'failing_examples': {},
     'passing_examples': [],
     'stale_examples': [],  # ones that did not need to be run due to md5sum
+    'run_stale_examples': False,
     'expected_failing_examples': set(),
     'thumbnail_size': (400, 280),  # Default CSS does 0.4 scaling (160, 112)
     'min_reported_time': 0,
@@ -83,28 +87,39 @@ DEFAULT_GALLERY_CONF = {
     'pypandoc': False,
     'remove_config_comments': False,
     'show_memory': False,
+    'show_signature': True,
     'junit': '',
     'log_level': {'backreference_missing': 'warning'},
     'inspect_global_variables': True,
     'css': _KNOWN_CSS,
     'matplotlib_animations': False,
+    'default_thumb_file': None,
+    'line_numbers': False,
 }
 
 logger = sphinx_compatibility.getLogger('sphinx-gallery')
 
 
-def parse_config(app):
-    """Process the Sphinx Gallery configuration"""
-    try:
-        plot_gallery = eval(app.builder.config.plot_gallery)
-    except TypeError:
-        plot_gallery = bool(app.builder.config.plot_gallery)
+def _bool_eval(x):
+    if isinstance(x, str):
+        try:
+            x = eval(x)
+        except TypeError:
+            pass
+    return bool(x)
+
+
+def parse_config(app, check_keys=True):
+    """Process the Sphinx Gallery configuration."""
+    plot_gallery = _bool_eval(app.builder.config.plot_gallery)
     src_dir = app.builder.srcdir
-    abort_on_example_error = app.builder.config.abort_on_example_error
+    abort_on_example_error = _bool_eval(
+        app.builder.config.abort_on_example_error)
     lang = app.builder.config.highlight_language
     gallery_conf = _complete_gallery_conf(
         app.config.sphinx_gallery_conf, src_dir, plot_gallery,
-        abort_on_example_error, lang, app.builder.name, app)
+        abort_on_example_error, lang, app.builder.name, app,
+        check_keys)
 
     # this assures I can call the config in other places
     app.config.sphinx_gallery_conf = gallery_conf
@@ -114,8 +129,21 @@ def parse_config(app):
 
 def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
                            abort_on_example_error, lang='python',
-                           builder_name='html', app=None):
+                           builder_name='html', app=None, check_keys=True):
     gallery_conf = copy.deepcopy(DEFAULT_GALLERY_CONF)
+    options = sorted(gallery_conf)
+    extra_keys = sorted(set(sphinx_gallery_conf) - set(options))
+    if extra_keys and check_keys:
+        msg = 'Unknown key(s) in sphinx_gallery_conf:\n'
+        for key in extra_keys:
+            options = get_close_matches(key, options, cutoff=0.66)
+            msg += repr(key)
+            if len(options) == 1:
+                msg += ', did you mean %r?' % (options[0],)
+            elif len(options) > 1:
+                msg += ', did you mean one of %r?' % (options,)
+            msg += '\n'
+        raise ConfigError(msg.strip())
     gallery_conf.update(sphinx_gallery_conf)
     if sphinx_gallery_conf.get('find_mayavi_figures', False):
         logger.warning(
@@ -126,23 +154,13 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
         gallery_conf['image_scrapers'] += ('mayavi',)
     gallery_conf.update(plot_gallery=plot_gallery)
     gallery_conf.update(abort_on_example_error=abort_on_example_error)
+    # XXX anything that can only be a bool (rather than str) should probably be
+    # evaluated this way as it allows setting via -D on the command line
+    for key in ('run_stale_examples',):
+        gallery_conf[key] = _bool_eval(gallery_conf[key])
     gallery_conf['src_dir'] = src_dir
     gallery_conf['app'] = app
 
-    if gallery_conf.get("mod_example_dir", False):
-        backreferences_warning = """\n========
-        Sphinx-Gallery found the configuration key 'mod_example_dir'. This
-        is deprecated, and you should now use the key 'backreferences_dir'
-        instead. Support for 'mod_example_dir' will be removed in a subsequent
-        version of Sphinx-Gallery. For more details, see the backreferences
-        documentation:
-
-        https://sphinx-gallery.github.io/configuration.html#references-to-examples"""  # noqa: E501
-        gallery_conf['backreferences_dir'] = gallery_conf['mod_example_dir']
-        logger.warning(
-            backreferences_warning,
-            type=DeprecationWarning)
-
     # Check capture_repr
     capture_repr = gallery_conf['capture_repr']
     supported_reprs = ['__repr__', '__str__', '_repr_html_']
@@ -322,6 +340,9 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
     if isinstance(backref, pathlib.Path):
         gallery_conf['backreferences_dir'] = str(backref)
 
+    # binder
+    gallery_conf['binder'] = check_binder_conf(gallery_conf['binder'])
+
     if not isinstance(gallery_conf['css'], (list, tuple)):
         raise ConfigError('gallery_conf["css"] must be list or tuple, got %r'
                           % (gallery_conf['css'],))
@@ -388,7 +409,7 @@ def _prepare_sphx_glr_dirs(gallery_conf, srcdir):
 def generate_gallery_rst(app):
     """Generate the Main examples gallery reStructuredText
 
-    Start the sphinx-gallery configuration and recursively scan the examples
+    Start the Sphinx-Gallery configuration and recursively scan the examples
     directories in order to populate the examples gallery
     """
     logger.info('generating gallery...', color='white')
@@ -441,7 +462,8 @@ def generate_gallery_rst(app):
                     gallery_dir, app.builder.srcdir)
                 fhindex.write(download_fhindex)
 
-            fhindex.write(SPHX_GLR_SIG)
+            if (app.config.sphinx_gallery_conf['show_signature']):
+                fhindex.write(SPHX_GLR_SIG)
         _replace_md5(index_rst_new, mode='t')
     _finalize_backreferences(seen_backrefs, gallery_conf)
 
@@ -642,7 +664,7 @@ def summarize_failing_examples(app, exception):
 
     # Under no-plot Examples are not run so nothing to summarize
     if not app.config.sphinx_gallery_conf['plot_gallery']:
-        logger.info('Sphinx-gallery gallery_conf["plot_gallery"] was '
+        logger.info('Sphinx-Gallery gallery_conf["plot_gallery"] was '
                     'False, so no examples were executed.', color='brown')
         return
 
@@ -677,7 +699,7 @@ def summarize_failing_examples(app, exception):
     n_good = len(gallery_conf['passing_examples'])
     n_tot = len(gallery_conf['failing_examples']) + n_good
     n_stale = len(gallery_conf['stale_examples'])
-    logger.info('\nSphinx-gallery successfully executed %d out of %d '
+    logger.info('\nSphinx-Gallery successfully executed %d out of %d '
                 'file%s subselected by:\n\n'
                 '    gallery_conf["filename_pattern"] = %r\n'
                 '    gallery_conf["ignore_pattern"]   = %r\n'
@@ -691,10 +713,13 @@ def summarize_failing_examples(app, exception):
                 color='brown')
 
     if fail_msgs:
-        raise ExtensionError(
-            "Here is a summary of the problems encountered "
-            "when running the examples\n\n" + "\n".join(fail_msgs) +
-            "\n" + "-" * 79)
+        fail_message = ("Here is a summary of the problems encountered "
+                        "when running the examples\n\n" +
+                        "\n".join(fail_msgs) + "\n" + "-" * 79)
+        if gallery_conf['only_warn_on_example_error']:
+            logger.warning(fail_message)
+        else:
+            raise ExtensionError(fail_message)
 
 
 def collect_gallery_files(examples_dirs, gallery_conf):
@@ -748,7 +773,7 @@ def get_default_config_value(key):
 
 
 def setup(app):
-    """Setup sphinx-gallery sphinx extension"""
+    """Setup Sphinx-Gallery sphinx extension"""
     sphinx_compatibility._app = app
 
     app.add_config_value('sphinx_gallery_conf', DEFAULT_GALLERY_CONF, 'html')
