@@ -10,8 +10,7 @@ example files.
 
 Files that generate images should start with 'plot'.
 """
-# Don't use unicode_literals here (be explicit with u"..." instead) otherwise
-# tricky errors come up with exec(code_blocks, ...) calls
+
 from __future__ import division, print_function, absolute_import
 from time import time
 import copy
@@ -25,6 +24,7 @@ import importlib
 from io import StringIO
 import os
 import re
+import stat
 from textwrap import indent
 import warnings
 from shutil import copyfile
@@ -33,9 +33,12 @@ import sys
 import traceback
 import codeop
 
+from sphinx.errors import ExtensionError
+
 from .scrapers import (save_figures, ImagePathIterator, clean_modules,
                        _find_image_ext)
-from .utils import replace_py_ipynb, scale_image, get_md5sum, _replace_md5
+from .utils import (replace_py_ipynb, scale_image, get_md5sum, _replace_md5,
+                    optipng)
 from . import glr_path_static
 from . import sphinx_compatibility
 from .backreferences import (_write_backreferences, _thumbnail_div,
@@ -53,18 +56,30 @@ logger = sphinx_compatibility.getLogger('sphinx-gallery')
 ###############################################################################
 
 
-class LoggingTee(object):
-    """A tee object to redirect streams to the logger"""
+class _LoggingTee(object):
+    """A tee object to redirect streams to the logger."""
 
-    def __init__(self, output_file, logger, src_filename):
-        self.output_file = output_file
+    def __init__(self, src_filename):
         self.logger = logger
         self.src_filename = src_filename
-        self.first_write = True
         self.logger_buffer = ''
+        self.set_std_and_reset_position()
+
+    def set_std_and_reset_position(self):
+        if not isinstance(sys.stdout, _LoggingTee):
+            self.origs = (sys.stdout, sys.stderr)
+        sys.stdout = sys.stderr = self
+        self.first_write = True
+        self.output = StringIO()
+        return self
+
+    def restore_std(self):
+        sys.stdout.flush()
+        sys.stderr.flush()
+        sys.stdout, sys.stderr = self.origs
 
     def write(self, data):
-        self.output_file.write(data)
+        self.output.write(data)
 
         if self.first_write:
             self.logger.verbose('Output from %s', self.src_filename,
@@ -85,14 +100,21 @@ class LoggingTee(object):
             self.logger.verbose('%s', line)
 
     def flush(self):
-        self.output_file.flush()
+        self.output.flush()
         if self.logger_buffer:
             self.logger.verbose('%s', self.logger_buffer)
             self.logger_buffer = ''
 
     # When called from a local terminal seaborn needs it in Python3
     def isatty(self):
-        return self.output_file.isatty()
+        return self.output.isatty()
+
+    # When called in gen_rst, conveniently use context managing
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type_, value, tb):
+        self.restore_std()
 
 
 ###############################################################################
@@ -116,8 +138,7 @@ SINGLE_IMAGE = """
     :class: sphx-glr-single-img
 """
 
-# This one could contain unicode
-CODE_OUTPUT = u""".. rst-class:: sphx-glr-script-out
+CODE_OUTPUT = """.. rst-class:: sphx-glr-script-out
 
  Out:
 
@@ -141,22 +162,24 @@ SPHX_GLR_SIG = """\n
 """
 
 # Header used to include raw html
-html_header = """.. only:: builder_html
+html_header = """.. raw:: html
 
-    .. raw:: html
-
-{0}\n        <br />\n        <br />"""
+    <div class="output_subarea output_html rendered_html output_result">
+{0}
+    </div>
+    <br />
+    <br />"""
 
 
 def codestr2rst(codestr, lang='python', lineno=None):
-    """Return reStructuredText code block from code string"""
+    """Return reStructuredText code block from code string."""
     if lineno is not None:
         # Sphinx only starts numbering from the first non-empty line.
         blank_lines = codestr.count('\n', 0, -len(codestr.lstrip()))
         lineno = '   :lineno-start: {0}\n'.format(lineno + blank_lines)
     else:
         lineno = ''
-    code_directive = "\n.. code-block:: {0}\n{1}\n".format(lang, lineno)
+    code_directive = ".. code-block:: {0}\n{1}\n".format(lang, lineno)
     indented_block = indent(codestr, ' ' * 4)
     return code_directive + indented_block
 
@@ -192,7 +215,7 @@ def extract_intro_and_title(filename, docstring):
     paragraphs = [p for p in paragraphs
                   if not p.startswith('.. ') and len(p) > 0]
     if len(paragraphs) == 0:
-        raise ValueError(
+        raise ExtensionError(
             "Example docstring should have a header for the example title. "
             "Please check the example file:\n {}\n".format(filename))
     # Title is the first paragraph with any ReSTructuredText title chars
@@ -204,7 +227,7 @@ def extract_intro_and_title(filename, docstring):
                       re.MULTILINE)
 
     if match is None:
-        raise ValueError(
+        raise ExtensionError(
             'Could not find a title in first paragraph:\n{}'.format(
                 title_paragraph))
     title = match.group(0).strip()
@@ -218,10 +241,10 @@ def extract_intro_and_title(filename, docstring):
     return intro, title
 
 
-def md5sum_is_current(src_file):
+def md5sum_is_current(src_file, mode='b'):
     """Checks whether src_file has the same md5 hash as the one on disk"""
 
-    src_md5 = get_md5sum(src_file)
+    src_md5 = get_md5sum(src_file, mode)
 
     src_md5_file = src_file + '.md5'
     if os.path.exists(src_md5_file):
@@ -233,7 +256,8 @@ def md5sum_is_current(src_file):
     return False
 
 
-def save_thumbnail(image_path_template, src_file, file_conf, gallery_conf):
+def save_thumbnail(image_path_template, src_file, script_vars, file_conf,
+                   gallery_conf):
     """Generate and Save the thumbnail image
 
     Parameters
@@ -242,6 +266,11 @@ def save_thumbnail(image_path_template, src_file, file_conf, gallery_conf):
         holds the template where to save and how to name the image
     src_file : str
         path to source python file
+    script_vars : dict
+        Configuration and run time variables
+    file_conf : dict
+        File-specific settings given in source file comments as:
+        ``# sphinx_gallery_<name> = <value>``
     gallery_conf : dict
         Sphinx-Gallery configuration dictionary
     """
@@ -261,9 +290,12 @@ def save_thumbnail(image_path_template, src_file, file_conf, gallery_conf):
         image_path = os.path.join(gallery_conf['src_dir'], thumbnail_path)
     else:
         if not isinstance(thumbnail_number, int):
-            raise TypeError(
+            raise ExtensionError(
                 'sphinx_gallery_thumbnail_number setting is not a number, '
                 'got %r' % (thumbnail_number,))
+        # negative index means counting from the last one
+        if thumbnail_number < 0:
+            thumbnail_number += len(script_vars["image_path_iterator"]) + 1
         image_path = image_path_template.format(thumbnail_number)
     del thumbnail_number, thumbnail_path, image_path_template
     thumbnail_image_path, ext = _find_image_ext(image_path)
@@ -278,14 +310,24 @@ def save_thumbnail(image_path_template, src_file, file_conf, gallery_conf):
         img = thumbnail_image_path
     elif not os.path.exists(thumb_file):
         # create something to replace the thumbnail
-        img = os.path.join(glr_path_static(), 'no_image.png')
-        img = gallery_conf.get("default_thumb_file", img)
+        default_thumb_path = gallery_conf.get("default_thumb_file")
+        if default_thumb_path is None:
+            default_thumb_path = os.path.join(
+                glr_path_static(),
+                'no_image.png',
+            )
+        img, ext = _find_image_ext(default_thumb_path)
     else:
         return
-    if ext == 'svg':
+    # update extension, since gallery_conf setting can be different
+    # from file_conf
+    thumb_file = '%s.%s' % (os.path.splitext(thumb_file)[0], ext)
+    if ext in ('svg', 'gif'):
         copyfile(img, thumb_file)
     else:
         scale_image(img, thumb_file, *gallery_conf["thumbnail_size"])
+        if 'thumbnails' in gallery_conf['compress_images']:
+            optipng(thumb_file, gallery_conf['compress_images_args'])
 
 
 def _get_readme(dir_, gallery_conf, raise_error=True):
@@ -296,7 +338,7 @@ def _get_readme(dir_, gallery_conf, raise_error=True):
             if os.path.isfile(fpth):
                 return fpth
     if raise_error:
-        raise FileNotFoundError(
+        raise ExtensionError(
             "Example directory {0} does not have a README file with one "
             "of the expected file extensions {1}. Please write one to "
             "introduce your gallery.".format(dir_, extensions))
@@ -336,12 +378,12 @@ def generate_dir_rst(src_dir, target_dir, gallery_conf, seen_backrefs):
         'generating gallery for %s... ' % build_target_dir,
         length=len(sorted_listdir))
     for fname in iterator:
-        intro, cost = generate_file_rst(
+        intro, title, cost = generate_file_rst(
             fname, target_dir, src_dir, gallery_conf, seen_backrefs)
         src_file = os.path.normpath(os.path.join(src_dir, fname))
         costs.append((cost, src_file))
         this_entry = _thumbnail_div(target_dir, gallery_conf['src_dir'],
-                                    fname, intro) + """
+                                    fname, intro, title) + """
 
 .. toctree::
    :hidden:
@@ -360,15 +402,47 @@ def generate_dir_rst(src_dir, target_dir, gallery_conf, seen_backrefs):
 
 
 def handle_exception(exc_info, src_file, script_vars, gallery_conf):
+    """Trim and format exception, maybe raise error, etc."""
     from .gen_gallery import _expected_failing_examples
     etype, exc, tb = exc_info
     stack = traceback.extract_tb(tb)
-    # Remove our code from traceback:
-    if isinstance(exc, SyntaxError):
-        # Remove one extra level through ast.parse.
-        stack = stack[2:]
-    else:
-        stack = stack[1:]
+    # The full traceback will look something like:
+    #
+    #   File "/home/larsoner/python/sphinx-gallery/sphinx_gallery/gen_rst.py...
+    #     mem_max, _ = gallery_conf['call_memory'](
+    #   File "/home/larsoner/python/sphinx-gallery/sphinx_gallery/gen_galler...
+    #     mem, out = memory_usage(func, max_usage=True, retval=True,
+    #   File "/home/larsoner/.local/lib/python3.8/site-packages/memory_profi...
+    #     returned = f(*args, **kw)
+    #   File "/home/larsoner/python/sphinx-gallery/sphinx_gallery/gen_rst.py...
+    #     exec(self.code, self.fake_main.__dict__)
+    #   File "/home/larsoner/python/sphinx-gallery/sphinx_gallery/tests/tiny...
+    #     raise RuntimeError('some error')
+    # RuntimeError: some error
+    #
+    # But we should trim these to just the relevant trace at the user level,
+    # so we inspect the traceback to find the start and stop points.
+    start = 0
+    stop = len(stack)
+    root = os.path.dirname(__file__) + os.sep
+    for ii, s in enumerate(stack, 1):
+        # Trim our internal stack
+        if s.filename.startswith(root + 'gen_gallery.py') and \
+                s.name == 'call_memory':
+            start = max(ii, start)
+        elif s.filename.startswith(root + 'gen_rst.py'):
+            # SyntaxError
+            if s.name == 'execute_code_block' and ('compile(' in s.line or
+                                                   'save_figures' in s.line):
+                start = max(ii, start)
+            # Any other error
+            elif s.name == '__call__':
+                start = max(ii, start)
+            # Our internal input() check
+            elif s.name == '_check_input' and ii == len(stack):
+                stop = ii - 1
+    stack = stack[start:stop]
+
     formatted_exception = 'Traceback (most recent call last):\n' + ''.join(
         traceback.format_list(stack) +
         traceback.format_exception_only(etype, exc))
@@ -445,50 +519,32 @@ class _exec_once(object):
                         sys.modules['__main__'] = old_main
 
 
-def _memory_usage(func, gallery_conf):
-    """Get memory usage of a function call."""
-    if gallery_conf['show_memory']:
-        from memory_profiler import memory_usage
-        assert callable(func)
-        mem, out = memory_usage(func, max_usage=True, retval=True,
-                                multiprocess=True)
-        try:
-            mem = mem[0]  # old MP always returned a list
-        except TypeError:  # 'float' object is not subscriptable
-            pass
-    else:
-        out = func()
-        mem = 0
-    return out, mem
-
-
 def _get_memory_base(gallery_conf):
     """Get the base amount of memory used by running a Python process."""
-    if not gallery_conf['show_memory'] or not gallery_conf['plot_gallery']:
-        memory_base = 0
+    if not gallery_conf['plot_gallery']:
+        return 0.
+    # There might be a cleaner way to do this at some point
+    from memory_profiler import memory_usage
+    if sys.platform in ('win32', 'darwin'):
+        sleep, timeout = (1, 2)
     else:
-        # There might be a cleaner way to do this at some point
-        from memory_profiler import memory_usage
-        if sys.platform in ('win32', 'darwin'):
-            sleep, timeout = (1, 2)
-        else:
-            sleep, timeout = (0.5, 1)
-        proc = subprocess.Popen(
-            [sys.executable, '-c',
-             'import time, sys; time.sleep(%s); sys.exit(0)' % sleep],
-            close_fds=True)
-        memories = memory_usage(proc, interval=1e-3, timeout=timeout)
-        kwargs = dict(timeout=timeout) if sys.version_info >= (3, 5) else {}
-        proc.communicate(**kwargs)
-        # On OSX sometimes the last entry can be None
-        memories = [mem for mem in memories if mem is not None] + [0.]
-        memory_base = max(memories)
+        sleep, timeout = (0.5, 1)
+    proc = subprocess.Popen(
+        [sys.executable, '-c',
+            'import time, sys; time.sleep(%s); sys.exit(0)' % sleep],
+        close_fds=True)
+    memories = memory_usage(proc, interval=1e-3, timeout=timeout)
+    kwargs = dict(timeout=timeout) if sys.version_info >= (3, 5) else {}
+    proc.communicate(**kwargs)
+    # On OSX sometimes the last entry can be None
+    memories = [mem for mem in memories if mem is not None] + [0.]
+    memory_base = max(memories)
     return memory_base
 
 
 def execute_code_block(compiler, block, example_globals,
                        script_vars, gallery_conf):
-    """Executes the code block of the example file"""
+    """Execute the code block of the example file."""
     if example_globals is None:  # testing shortcut
         example_globals = script_vars['fake_main'].__dict__
     blabel, bcontent, lineno = block
@@ -498,18 +554,23 @@ def execute_code_block(compiler, block, example_globals,
 
     cwd = os.getcwd()
     # Redirect output to stdout and
-    orig_stdout, orig_stderr = sys.stdout, sys.stderr
+
     src_file = script_vars['src_file']
+    logging_tee = _check_reset_logging_tee(src_file)
+    assert isinstance(logging_tee, _LoggingTee)
 
     # First cd in the original example dir, so that any file
     # created by the example get created in this directory
 
-    captured_std = StringIO()
     os.chdir(os.path.dirname(src_file))
 
     sys_path = copy.deepcopy(sys.path)
     sys.path.append(os.getcwd())
-    sys.stdout = sys.stderr = LoggingTee(captured_std, logger, src_file)
+
+    # Save figures unless there is a `sphinx_gallery_defer_figures` flag
+    match = re.search(r'^[\ \t]*#\s*sphinx_gallery_defer_figures[\ \t]*\n?',
+                      bcontent, re.MULTILINE)
+    need_save_figures = match is None
 
     try:
         dont_inherit = 1
@@ -518,8 +579,8 @@ def execute_code_block(compiler, block, example_globals,
         else:
             ast_Module = ast.Module
         code_ast = ast_Module([bcontent])
-        code_ast = compile(bcontent, src_file, 'exec',
-                           ast.PyCF_ONLY_AST | compiler.flags, dont_inherit)
+        flags = ast.PyCF_ONLY_AST | compiler.flags
+        code_ast = compile(bcontent, src_file, 'exec', flags, dont_inherit)
         ast.increment_lineno(code_ast, lineno - 1)
         # capture output if last line is expression
         is_last_expr = False
@@ -527,45 +588,45 @@ def execute_code_block(compiler, block, example_globals,
             is_last_expr = True
             last_val = code_ast.body.pop().value
             # exec body minus last expression
-            _, mem_body = _memory_usage(
+            mem_body, _ = gallery_conf['call_memory'](
                 _exec_once(
                     compiler(code_ast, src_file, 'exec'),
-                    script_vars['fake_main']),
-                gallery_conf)
+                    script_vars['fake_main']))
             # exec last expression, made into assignment
             body = [ast.Assign(
                 targets=[ast.Name(id='___', ctx=ast.Store())], value=last_val)]
             last_val_ast = ast_Module(body=body)
             ast.fix_missing_locations(last_val_ast)
-            _, mem_last = _memory_usage(
+            mem_last, _ = gallery_conf['call_memory'](
                 _exec_once(
                     compiler(last_val_ast, src_file, 'exec'),
-                    script_vars['fake_main']),
-                gallery_conf)
+                    script_vars['fake_main']))
             # capture the assigned variable
             ___ = example_globals['___']
             mem_max = max(mem_body, mem_last)
         else:
-            _, mem_max = _memory_usage(
+            mem_max, _ = gallery_conf['call_memory'](
                 _exec_once(
                     compiler(code_ast, src_file, 'exec'),
-                    script_vars['fake_main']),
-                gallery_conf)
+                    script_vars['fake_main']))
         script_vars['memory_delta'].append(mem_max)
+        # This should be inside the try block, e.g., in case of a savefig error
+        logging_tee.restore_std()
+        if need_save_figures:
+            need_save_figures = False
+            images_rst = save_figures(block, script_vars, gallery_conf)
+        else:
+            images_rst = u''
     except Exception:
-        sys.stdout.flush()
-        sys.stderr.flush()
-        sys.stdout, sys.stderr = orig_stdout, orig_stderr
+        logging_tee.restore_std()
         except_rst = handle_exception(sys.exc_info(), src_file, script_vars,
                                       gallery_conf)
         code_output = u"\n{0}\n\n\n\n".format(except_rst)
         # still call this even though we won't use the images so that
         # figures are closed
-        save_figures(block, script_vars, gallery_conf)
+        if need_save_figures:
+            save_figures(block, script_vars, gallery_conf)
     else:
-        sys.stdout.flush()
-        sys.stderr.flush()
-        sys.stdout, orig_stderr = orig_stdout, orig_stderr
         sys.path = sys_path
         os.chdir(cwd)
 
@@ -592,7 +653,7 @@ def execute_code_block(compiler, block, example_globals,
                     else:
                         if isinstance(last_repr, str):
                             break
-        captured_std = captured_std.getvalue().expandtabs()
+        captured_std = logging_tee.output.getvalue().expandtabs()
         # normal string output
         if repr_meth in ['__repr__', '__str__'] and last_repr:
             captured_std = u"{0}\n{1}".format(captured_std, last_repr)
@@ -600,10 +661,9 @@ def execute_code_block(compiler, block, example_globals,
             captured_std = CODE_OUTPUT.format(indent(captured_std, u' ' * 4))
         else:
             captured_std = ''
-        images_rst = save_figures(block, script_vars, gallery_conf)
         # give html output its own header
         if repr_meth == '_repr_html_':
-            captured_html = html_header.format(indent(last_repr, u' ' * 8))
+            captured_html = html_header.format(indent(last_repr, u' ' * 4))
         else:
             captured_html = ''
         code_output = u"\n{0}\n\n{1}\n{2}\n\n".format(
@@ -612,11 +672,22 @@ def execute_code_block(compiler, block, example_globals,
     finally:
         os.chdir(cwd)
         sys.path = sys_path
-        sys.stdout, sys.stderr = orig_stdout, orig_stderr
+        logging_tee.restore_std()
 
     return code_output
 
 
+def _check_reset_logging_tee(src_file):
+    # Helper to deal with our tests not necessarily calling execute_script
+    # but rather execute_code_block directly
+    if isinstance(sys.stdout, _LoggingTee):
+        logging_tee = sys.stdout
+    else:
+        logging_tee = _LoggingTee(src_file)
+    logging_tee.set_std_and_reset_position()
+    return logging_tee
+
+
 def executable_script(src_file, gallery_conf):
     """Validate if script has to be run according to gallery configuration
 
@@ -640,6 +711,11 @@ def executable_script(src_file, gallery_conf):
     return execute
 
 
+def _check_input(prompt=None):
+    raise ExtensionError(
+        'Cannot use input() builtin function in Sphinx-Gallery examples')
+
+
 def execute_script(script_blocks, script_vars, gallery_conf):
     """Execute and capture output from python script already in block structure
 
@@ -679,6 +755,8 @@ def execute_script(script_blocks, script_vars, gallery_conf):
         # want to print it
         '__doc__': '',
         # Don't ever support __file__: Issues #166 #212
+        # Don't let them use input()
+        'input': _check_input,
     })
     script_vars['example_globals'] = example_globals
 
@@ -688,9 +766,9 @@ def execute_script(script_blocks, script_vars, gallery_conf):
         # https://github.com/sphinx-gallery/sphinx-gallery/pull/252
         # for more details.
         sys.argv[0] = script_vars['src_file']
-        sys.argv[1:] = []
+        sys.argv[1:] = gallery_conf['reset_argv'](gallery_conf, script_vars)
         gc.collect()
-        _, memory_start = _memory_usage(lambda: None, gallery_conf)
+        memory_start, _ = gallery_conf['call_memory'](lambda: None)
     else:
         memory_start = 0.
 
@@ -699,10 +777,12 @@ def execute_script(script_blocks, script_vars, gallery_conf):
     # include at least one entry to avoid max() ever failing
     script_vars['memory_delta'] = [memory_start]
     script_vars['fake_main'] = fake_main
-    output_blocks = [execute_code_block(compiler, block,
-                                        example_globals,
-                                        script_vars, gallery_conf)
-                     for block in script_blocks]
+    output_blocks = list()
+    with _LoggingTee(script_vars.get('src_file', '')) as logging_tee:
+        for block in script_blocks:
+            logging_tee.set_std_and_reset_position()
+            output_blocks.append(execute_code_block(
+                compiler, block, example_globals, script_vars, gallery_conf))
     time_elapsed = time() - t_start
     sys.argv = argv_orig
     script_vars['memory_delta'] = max(script_vars['memory_delta'])
@@ -711,7 +791,7 @@ def execute_script(script_blocks, script_vars, gallery_conf):
         # Write md5 checksum if the example was meant to run (no-plot
         # shall not cache md5sum) and has built correctly
         with open(script_vars['target_file'] + '.md5', 'w') as file_checksum:
-            file_checksum.write(get_md5sum(script_vars['target_file']))
+            file_checksum.write(get_md5sum(script_vars['target_file'], 't'))
         gallery_conf['passing_examples'].append(script_vars['src_file'])
 
     return output_blocks, time_elapsed
@@ -744,7 +824,7 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
     seen_backrefs = set() if seen_backrefs is None else seen_backrefs
     src_file = os.path.normpath(os.path.join(src_dir, fname))
     target_file = os.path.join(target_dir, fname)
-    _replace_md5(src_file, target_file, 'copy')
+    _replace_md5(src_file, target_file, 'copy', mode='t')
 
     file_conf, script_blocks, node = split_code_and_text_blocks(
         src_file, return_node=True)
@@ -753,10 +833,15 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
 
     executable = executable_script(src_file, gallery_conf)
 
-    if md5sum_is_current(target_file):
+    if md5sum_is_current(target_file, mode='t'):
+        do_return = True
         if executable:
-            gallery_conf['stale_examples'].append(target_file)
-        return intro, (0, 0)
+            if gallery_conf['run_stale_examples']:
+                do_return = False
+            else:
+                gallery_conf['stale_examples'].append(target_file)
+        if do_return:
+            return intro, title, (0, 0)
 
     image_dir = os.path.join(target_dir, 'images')
     if not os.path.exists(image_dir):
@@ -772,12 +857,6 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
         'src_file': src_file,
         'target_file': target_file}
 
-    if gallery_conf['remove_config_comments']:
-        script_blocks = [
-            (label, remove_config_comments(content), line_number)
-            for label, content, line_number in script_blocks
-        ]
-
     if executable:
         clean_modules(gallery_conf, fname)
     output_blocks, time_elapsed = execute_script(script_blocks,
@@ -786,6 +865,18 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
 
     logger.debug("%s ran in : %.2g seconds\n", src_file, time_elapsed)
 
+    if gallery_conf['remove_config_comments']:
+        script_blocks = [
+            (label, remove_config_comments(content), line_number)
+            for label, content, line_number in script_blocks
+        ]
+
+    # Remove final empty block, which can occur after config comments
+    # are removed
+    if script_blocks[-1][1].isspace():
+        script_blocks = script_blocks[:-1]
+        output_blocks = output_blocks[:-1]
+
     example_rst = rst_blocks(script_blocks, output_blocks,
                              file_conf, gallery_conf)
     memory_used = gallery_conf['memory_base'] + script_vars['memory_delta']
@@ -794,12 +885,13 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
     save_rst_example(example_rst, target_file, time_elapsed, memory_used,
                      gallery_conf)
 
-    save_thumbnail(image_path_template, src_file, file_conf, gallery_conf)
+    save_thumbnail(image_path_template, src_file, script_vars, file_conf,
+                   gallery_conf)
 
-    example_nb = jupyter_notebook(script_blocks, gallery_conf)
+    example_nb = jupyter_notebook(script_blocks, gallery_conf, target_dir)
     ipy_fname = replace_py_ipynb(target_file) + '.new'
     save_notebook(example_nb, ipy_fname)
-    _replace_md5(ipy_fname)
+    _replace_md5(ipy_fname, mode='t')
 
     # Write names
     if gallery_conf['inspect_global_variables']:
@@ -818,13 +910,39 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
                    if cobj['module'].startswith(gallery_conf['doc_module']))
     # Write backreferences
     _write_backreferences(backrefs, seen_backrefs, gallery_conf, target_dir,
-                          fname, intro)
+                          fname, intro, title)
 
-    return intro, (time_elapsed, memory_used)
+    return intro, title, (time_elapsed, memory_used)
+
+
+EXAMPLE_HEADER = """
+.. DO NOT EDIT.
+.. THIS FILE WAS AUTOMATICALLY GENERATED BY SPHINX-GALLERY.
+.. TO MAKE CHANGES, EDIT THE SOURCE PYTHON FILE:
+.. "{0}"
+.. LINE NUMBERS ARE GIVEN BELOW.
+
+.. only:: html
+
+    .. note::
+        :class: sphx-glr-download-link-note
+
+        Click :ref:`here <sphx_glr_download_{1}>`
+        to download the full example code{2}
+
+.. rst-class:: sphx-glr-example-title
+
+.. _sphx_glr_{1}:
+
+"""
+RST_BLOCK_HEADER = """\
+.. GENERATED FROM PYTHON SOURCE LINES {0}-{1}
+
+"""
 
 
 def rst_blocks(script_blocks, output_blocks, file_conf, gallery_conf):
-    """Generates the rst string containing the script prose, code and output
+    """Generate the rst string containing the script prose, code and output.
 
     Parameters
     ----------
@@ -850,9 +968,14 @@ def rst_blocks(script_blocks, output_blocks, file_conf, gallery_conf):
     # A simple example has two blocks: one for the
     # example introduction/explanation and one for the code
     is_example_notebook_like = len(script_blocks) > 2
-    example_rst = u""  # there can be unicode content
-    for (blabel, bcontent, lineno), code_output in \
-            zip(script_blocks, output_blocks):
+    example_rst = ""
+    for bi, ((blabel, bcontent, lineno), code_output) in \
+            enumerate(zip(script_blocks, output_blocks)):
+        # do not add comment to the title block, otherwise the linking does
+        # not work properly
+        if bi > 0:
+            example_rst += RST_BLOCK_HEADER.format(
+                lineno, lineno + bcontent.count('\n'))
         if blabel == 'code':
 
             if not file_conf.get('line_numbers',
@@ -894,21 +1017,15 @@ def save_rst_example(example_rst, example_file, time_elapsed,
         Sphinx-Gallery configuration dictionary
     """
 
-    ref_fname = os.path.relpath(example_file, gallery_conf['src_dir'])
-    ref_fname = ref_fname.replace(os.path.sep, "_")
+    example_fname = os.path.relpath(example_file, gallery_conf['src_dir'])
+    ref_fname = example_fname.replace(os.path.sep, "_")
 
     binder_conf = check_binder_conf(gallery_conf.get('binder'))
 
     binder_text = (" or to run this example in your browser via Binder"
                    if len(binder_conf) else "")
-    example_rst = (".. only:: html\n\n"
-                   "    .. note::\n"
-                   "        :class: sphx-glr-download-link-note\n\n"
-                   "        Click :ref:`here <sphx_glr_download_{0}>` "
-                   "    to download the full example code{1}\n"
-                   "    .. rst-class:: sphx-glr-example-title\n\n"
-                   "    .. _sphx_glr_{0}:\n\n"
-                   ).format(ref_fname, binder_text) + example_rst
+    example_rst = EXAMPLE_HEADER.format(
+        example_fname, ref_fname, binder_text) + example_rst
 
     if time_elapsed >= gallery_conf["min_reported_time"]:
         time_m, time_s = divmod(time_elapsed, 60)
@@ -933,6 +1050,10 @@ def save_rst_example(example_rst, example_file, time_elapsed,
     write_file_new = re.sub(r'\.py$', '.rst.new', example_file)
     with codecs.open(write_file_new, 'w', encoding="utf-8") as f:
         f.write(example_rst)
+    # make it read-only so that people don't try to edit it
+    mode = os.stat(write_file_new).st_mode
+    ro_mask = 0x777 ^ (stat.S_IWRITE | stat.S_IWGRP | stat.S_IWOTH)
+    os.chmod(write_file_new, mode & ro_mask)
     # in case it wasn't in our pattern, only replace the file if it's
     # still stale.
-    _replace_md5(write_file_new)
+    _replace_md5(write_file_new, mode='t')
