@@ -15,10 +15,6 @@ import shutil
 import sys
 import time
 
-import numpy as np
-from numpy.testing import assert_allclose
-
-from sphinx_gallery.__init__ import glr_path_static
 from sphinx.application import Sphinx
 from sphinx.errors import ExtensionError
 from sphinx.util.docutils import docutils_namespace
@@ -27,7 +23,8 @@ from sphinx_gallery.utils import (_get_image, scale_image, _has_optipng,
 
 import pytest
 
-N_TOT = 11
+N_TOT = 13
+
 N_FAILING = 2
 N_GOOD = N_TOT - N_FAILING
 N_RST = 15 + N_TOT
@@ -36,6 +33,9 @@ N_RST = '(%s|%s)' % (N_RST, N_RST - 1)  # AppVeyor weirdness
 
 @pytest.fixture(scope='module')
 def sphinx_app(tmpdir_factory, req_mpl, req_pil):
+    # Skip if numpy not installed
+    pytest.importorskip("numpy")
+
     temp_dir = (tmpdir_factory.getbasetemp() / 'root').strpath
     src_dir = op.join(op.dirname(__file__), 'tinybuild')
 
@@ -162,9 +162,10 @@ def test_run_sphinx(sphinx_app):
 
 def test_thumbnail_path(sphinx_app, tmpdir):
     """Test sphinx_gallery_thumbnail_path."""
+    import numpy as np
     # Make sure our thumbnail matches what it should be
     fname_orig = op.join(
-        sphinx_app.srcdir, '_static', 'demo.png')
+        sphinx_app.srcdir, '_static_nonstandard', 'demo.png')
     fname_thumb = op.join(
         sphinx_app.outdir, '_images',
         'sphx_glr_plot_second_future_imports_thumb.png')
@@ -180,6 +181,29 @@ def test_thumbnail_path(sphinx_app, tmpdir):
     assert corr > 0.99
 
 
+def test_negative_thumbnail_config(sphinx_app, tmpdir):
+    """Test 'sphinx_gallery_thumbnail_number' config works correctly for
+    negative numbers."""
+    import numpy as np
+    # Make sure our thumbnail is the 2nd (last) image
+    fname_orig = op.join(
+        sphinx_app.outdir, '_images',
+        'sphx_glr_plot_matplotlib_alt_002.png')
+    fname_thumb = op.join(
+        sphinx_app.outdir, '_images',
+        'sphx_glr_plot_matplotlib_alt_thumb.png')
+    fname_new = str(tmpdir.join('new.png'))
+    scale_image(fname_orig, fname_new,
+                *sphinx_app.config.sphinx_gallery_conf["thumbnail_size"])
+    Image = _get_image()
+    orig = np.asarray(Image.open(fname_thumb))
+    new = np.asarray(Image.open(fname_new))
+    assert new.shape[:2] == orig.shape[:2]
+    assert new.shape[2] in (3, 4)  # optipng can strip the alpha channel
+    corr = np.corrcoef(new[..., :3].ravel(), orig[..., :3].ravel())[0, 1]
+    assert corr > 0.99
+
+
 def test_command_line_args_img(sphinx_app):
     generated_examples_dir = op.join(sphinx_app.outdir, 'auto_examples')
     thumb_fname = '../_images/sphx_glr_plot_command_line_args_thumb.png'
@@ -221,6 +245,16 @@ def test_image_formats(sphinx_app):
             assert extra in html
 
 
+def test_repr_html_classes(sphinx_app):
+    """Test appropriate _repr_html_ classes."""
+    example_file = op.join(
+        sphinx_app.outdir, 'auto_examples', 'plot_repr.html')
+    with codecs.open(example_file, 'r', 'utf-8') as fid:
+        lines = fid.read()
+    assert 'div class="output_subarea output_html rendered_html output_result"' in lines  # noqa: E501
+    assert 'gallery-rendered-html.css' in lines
+
+
 def test_embed_links_and_styles(sphinx_app):
     """Test that links and styles are embedded properly in doc."""
     out_dir = sphinx_app.outdir
@@ -241,7 +275,7 @@ def test_embed_links_and_styles(sphinx_app):
     assert 'class="sphx-glr-backref-module-numpy sphx-glr-backref-type-py-function">' in lines  # noqa: E501
     assert '#module-matplotlib.pyplot' in lines
     assert 'pyplot.html' in lines
-    assert 'matplotlib.figure.Figure.html#matplotlib.figure.Figure.tight_layout' in lines  # noqa: E501
+    assert '.html#matplotlib.figure.Figure.tight_layout' in lines
     assert 'matplotlib.axes.Axes.plot.html#matplotlib.axes.Axes.plot' in lines
     assert 'matplotlib_configuration_api.html#matplotlib.RcParams' in lines
     assert 'stdtypes.html#list' in lines
@@ -286,7 +320,7 @@ def test_embed_links_and_styles(sphinx_app):
     )
     assert dummy_class_meth is not None
     # property (Sphinx 2+ calls it a method rather than attribute, so regex)
-    dummy_class_prop = re.compile(r'sphinx_gallery.backreferences.html#sphinx[_,-]gallery[.,-]backreferences[.,-][D,d]ummy[C,c]lass[.,-]prop" title="sphinx_gallery.backreferences.DummyClass.prop" class="sphx-glr-backref-module-sphinx_gallery-backreferences sphx-glr-backref-type-py-(attribute|method)"><span class="n">dc</span><span class="o">.</span><span class="n">prop</span>')  # noqa: E501
+    dummy_class_prop = re.compile(r'sphinx_gallery.backreferences.html#sphinx[_,-]gallery[.,-]backreferences[.,-][D,d]ummy[C,c]lass[.,-]prop" title="sphinx_gallery.backreferences.DummyClass.prop" class="sphx-glr-backref-module-sphinx_gallery-backreferences sphx-glr-backref-type-py-(attribute|method|property)"><span class="n">dc</span><span class="o">.</span><span class="n">prop</span>')  # noqa: E501
     assert dummy_class_prop.search(lines) is not None
 
     try:
@@ -366,14 +400,22 @@ def test_logging_std_nested(sphinx_app):
 
 
 def _assert_mtimes(list_orig, list_new, different=(), ignore=()):
+    """Assert that the correct set of files were changed based on mtime."""
+    import numpy as np
+    from numpy.testing import assert_allclose
+
     assert ([op.basename(x) for x in list_orig] ==
             [op.basename(x) for x in list_new])
     for orig, new in zip(list_orig, list_new):
-        if op.basename(orig) in different:
+        check_name = op.splitext(op.basename(orig))[0]
+        if check_name.endswith('_codeobj'):
+            check_name = check_name[:-8]
+        if check_name in different:
             assert np.abs(op.getmtime(orig) - op.getmtime(new)) > 0.1
-        elif op.basename(orig) not in ignore:
+        elif check_name not in ignore:
             assert_allclose(op.getmtime(orig), op.getmtime(new),
-                            atol=1e-3, rtol=1e-20, err_msg=op.basename(orig))
+                            atol=1e-3, rtol=1e-20,
+                            err_msg=op.basename(orig))
 
 
 def test_rebuild(tmpdir_factory, sphinx_app):
@@ -445,8 +487,13 @@ def test_rebuild(tmpdir_factory, sphinx_app):
         new_app.build(False, [])
     status = new_app._status.getvalue()
     lines = [line for line in status.split('\n') if '0 removed' in line]
-    assert re.match('.*[0|1] added, [1-9] changed, 0 removed$.*',
-                    status, re.MULTILINE | re.DOTALL) is not None, lines
+    # XXX on AppVeyor this can be 13
+    if sys.platform.startswith('win'):
+        assert re.match('.*[0|1] added, [1-9][0-3]? changed, 0 removed$.*',
+                        status, re.MULTILINE | re.DOTALL) is not None, lines
+    else:
+        assert re.match('.*[0|1] added, [1-9] changed, 0 removed$.*',
+                        status, re.MULTILINE | re.DOTALL) is not None, lines
     want = ('.*executed 0 out of %s.*after excluding %s files.*based on MD5.*'
             % (N_FAILING, N_GOOD))
     assert re.match(want, status, re.MULTILINE | re.DOTALL) is not None
@@ -491,9 +538,9 @@ def test_rebuild(tmpdir_factory, sphinx_app):
         # these two should almost always be different, but in case we
         # get extremely unlucky and have identical run times
         # on the one script that gets re-run (because it's a fail)...
-        'sg_execution_times.rst',
-        'plot_future_imports_broken.rst',
-        'plot_scraper_broken.rst'
+        'sg_execution_times',
+        'plot_future_imports_broken',
+        'plot_scraper_broken'
     )
     _assert_mtimes(generated_rst_0, generated_rst_1, ignore=ignore)
 
@@ -507,32 +554,66 @@ def test_rebuild(tmpdir_factory, sphinx_app):
     _assert_mtimes(copied_ipy_0, copied_ipy_1)
 
     #
-    # run a third time, changing one file
+    # run a third and a fourth time, changing one file or running one stale
     #
 
+    for how in ('run_stale', 'modify'):
+        # modify must be last as this rerun setting tries to run the
+        # broken example (subsequent tests depend on it)
+        _rerun(how, src_dir, conf_dir, out_dir, toctrees_dir,
+               generated_modules_0, generated_backrefs_0, generated_rst_0,
+               generated_pickle_0, copied_py_0, copied_ipy_0)
+
+
+def _rerun(how, src_dir, conf_dir, out_dir, toctrees_dir,
+           generated_modules_0, generated_backrefs_0, generated_rst_0,
+           generated_pickle_0, copied_py_0, copied_ipy_0):
+    """Rerun the sphinx build and check that the right files were changed."""
     time.sleep(0.1)
-    fname = op.join(src_dir, 'examples', 'plot_numpy_matplotlib.py')
-    with codecs.open(fname, 'r', 'utf-8') as fid:
-        lines = fid.readlines()
-    with codecs.open(fname, 'w', 'utf-8') as fid:
-        for line in lines:
-            if line.startswith('FYI this'):
-                line = 'A ' + line
-            fid.write(line)
+    confoverrides = dict()
+    if how == 'modify':
+        fname = op.join(src_dir, 'examples', 'plot_numpy_matplotlib.py')
+        with codecs.open(fname, 'r', 'utf-8') as fid:
+            lines = fid.readlines()
+        with codecs.open(fname, 'w', 'utf-8') as fid:
+            for line in lines:
+                if 'FYI this' in line:
+                    line = 'A ' + line
+                fid.write(line)
+        out_of, excluding = N_FAILING + 1, N_GOOD - 1
+        n_stale = N_GOOD - 1
+    else:
+        assert how == 'run_stale'
+        confoverrides['sphinx_gallery_conf.run_stale_examples'] = 'True'
+        confoverrides['sphinx_gallery_conf.filename_pattern'] = 'plot_numpy_ma'
+        out_of, excluding = 1, 0
+        n_stale = 0
     with docutils_namespace():
         new_app = Sphinx(src_dir, conf_dir, out_dir, toctrees_dir,
-                         buildername='html', status=StringIO())
+                         buildername='html', status=StringIO(),
+                         confoverrides=confoverrides)
         new_app.build(False, [])
     status = new_app._status.getvalue()
-    n = '[' + '|'.join(str(x + N_FAILING) for x in range(4)) + ']'
     lines = [line for line in status.split('\n') if 'source files tha' in line]
-    want = '.*targets for %s source files that are out of date$.*' % n
-    assert re.match(want, status, re.MULTILINE | re.DOTALL) is not None, lines
+    lines = '\n'.join([how] + lines)
+    flags = re.MULTILINE | re.DOTALL
+    # for some reason, setting "confoverrides" above causes Sphinx to show
+    # all targets out of date, even though they haven't been modified...
+    want = '.*targets for %s source files that are out of date$.*' % N_RST
+    assert re.match(want, status, flags) is not None, lines
+    # ... but then later detects that only some have actually changed:
+    # Linux: 8 changed when how='run_stale', 9 when how='modify'.
+    # Windows: always 9 for some reason
+    lines = [line for line in status.split('\n') if 'changed,' in line]
+    lines = '\n'.join([how] + lines)
+    n_ch = '[8|9]'
+    want = '.*updating environment:.*0 added, %s changed, 0 removed.*' % n_ch
+    assert re.match(want, status, flags) is not None, lines
     want = ('.*executed 1 out of %s.*after excluding %s files.*based on MD5.*'
-            % (N_FAILING + 1, N_GOOD - 1,))
-    assert re.match(want, status, re.MULTILINE | re.DOTALL) is not None
-    n_stale = len(new_app.config.sphinx_gallery_conf['stale_examples'])
-    assert n_stale == N_GOOD - 1
+            % (out_of, excluding))
+    assert re.match(want, status, flags) is not None
+    got_stale = len(new_app.config.sphinx_gallery_conf['stale_examples'])
+    assert got_stale == n_stale
     assert op.isfile(op.join(new_app.outdir, '_images',
                              'sphx_glr_plot_numpy_matplotlib_001.png'))
 
@@ -568,32 +649,30 @@ def test_rebuild(tmpdir_factory, sphinx_app):
     _assert_mtimes(generated_backrefs_0, generated_backrefs_1)
 
     # generated RST files
-    different = (
-        'plot_numpy_matplotlib.rst',
-    )
+    different = ('plot_numpy_matplotlib',)
     ignore = (
         # this one should almost always be different, but in case we
         # get extremely unlucky and have identical run times
         # on the one script above that changes...
-        'sg_execution_times.rst',
+        'sg_execution_times',
         # this one will not change even though it was retried
-        'plot_future_imports_broken.rst',
-        'plot_scraper_broken.rst',
+        'plot_future_imports_broken',
+        'plot_scraper_broken',
     )
-    if not sys.platform.startswith('win'):  # not reliable on Windows
+    # not reliable on Windows and one Ubuntu run
+    bad = sys.platform.startswith('win') or os.getenv('BAD_MTIME', '0') == '1'
+    if not bad:
         _assert_mtimes(generated_rst_0, generated_rst_1, different, ignore)
 
         # mtimes for pickles
-        _assert_mtimes(generated_pickle_0, generated_pickle_1,
-                       different=('plot_numpy_matplotlib.codeobj.pickle'))
+        use_different = () if how == 'run_stale' else different
+        _assert_mtimes(generated_pickle_0, generated_pickle_1, ignore=ignore)
 
         # mtimes for .py files (gh-395)
-        _assert_mtimes(copied_py_0, copied_py_1,
-                       different=('plot_numpy_matplotlib.py'))
+        _assert_mtimes(copied_py_0, copied_py_1, different=use_different)
 
         # mtimes for .ipynb files
-        _assert_mtimes(copied_ipy_0, copied_ipy_1,
-                       different=('plot_numpy_matplotlib.ipynb'))
+        _assert_mtimes(copied_ipy_0, copied_ipy_1, different=use_different)
 
 
 @pytest.mark.parametrize('name, want', [
@@ -806,6 +885,7 @@ def test_jupyter_notebook_pandoc(sphinx_app):
 
 
 def test_md5_hash(sphinx_app):
+    """Test MD5 hashing."""
     src_dir = sphinx_app.srcdir
     fname = op.join(src_dir, 'auto_examples', 'plot_log.py.md5')
     expected_md5 = '0edc2de97f96f3b55f8b4a21994931a8'
@@ -816,9 +896,29 @@ def test_md5_hash(sphinx_app):
 
 
 def test_binder_logo_exists(sphinx_app):
-    """Check that the binder logo path is correct."""
-    src_dir = sphinx_app.srcdir
-    binder_logo = os.path.join(glr_path_static(), 'binder_badge_logo.svg')
-    binder_logo_path = os.path.relpath(binder_logo, src_dir)
+    """Test that the binder logo path is correct."""
+    root = op.join(sphinx_app.outdir, 'auto_examples')
+    with codecs.open(op.join(root, 'plot_svg.html'), 'r', 'utf-8') as fid:
+        html = fid.read()
+    path = re.match(r'.*<img alt="Launch binder" src="(.*)" width=.*\/>.*',
+                    html, re.DOTALL)
+    assert path is not None
+    path = path.groups()[0]
+    img_fname = op.abspath(op.join(root, path))
+    assert 'binder_badge_logo' in img_fname  # can have numbers appended
+    assert op.isfile(img_fname)
+    assert 'https://mybinder.org/v2/gh/sphinx-gallery/sphinx-gallery.github.io/master?urlpath=lab/tree/notebooks/auto_examples/plot_svg.ipynb' in html  # noqa: E501
 
-    assert op.isfile('/{}'.format(binder_logo_path))
+
+def test_defer_figures(sphinx_app):
+    """Test the deferring of figures."""
+    root = op.join(sphinx_app.outdir, 'auto_examples')
+    fname = op.join(root, 'plot_defer_figures.html')
+    with codecs.open(fname, 'r', 'utf-8') as fid:
+        html = fid.read()
+
+    # The example has two code blocks with plotting commands, but the first
+    # block has the flag ``sphinx_gallery_defer_figures``.  Thus, there should
+    # be only one image, not two, in the output.
+    assert '../_images/sphx_glr_plot_defer_figures_001.png' in html
+    assert '../_images/sphx_glr_plot_defer_figures_002.png' not in html
