@@ -37,7 +37,8 @@ CONTENT = [
     'And this is a second paragraph',
     '"""',
     '',
-    '# sphinx_gallery_thumbnail_number = 1'
+    '# sphinx_gallery_thumbnail_number = 1',
+    '# sphinx_gallery_defer_figures',
     '# and now comes the module code',
     'import logging',
     'import sys',
@@ -130,6 +131,7 @@ def test_rst_block_after_docstring(gallery_conf, tmpdir):
         f.write('\n'.join(['"Docstring"',
                            '####################',
                            '# Paragraph 1',
+                           '# is long.',
                            '',
                            '#%%',
                            '# Paragraph 2',
@@ -152,16 +154,24 @@ def test_rst_block_after_docstring(gallery_conf, tmpdir):
         blocks, script_vars, gallery_conf)
 
     example_rst = sg.rst_blocks(blocks, output_blocks, file_conf, gallery_conf)
-    assert example_rst == '\n'.join([
-        'Docstring',
-        '',
-        'Paragraph 1',
-        '',
-        'Paragraph 2',
-        '',
-        'Paragraph 3',
-        '',
-        ''])
+    want_rst = """\
+Docstring
+
+.. GENERATED FROM PYTHON SOURCE LINES 3-5
+
+Paragraph 1
+is long.
+
+.. GENERATED FROM PYTHON SOURCE LINES 7-8
+
+Paragraph 2
+
+.. GENERATED FROM PYTHON SOURCE LINES 10-11
+
+Paragraph 3
+
+"""
+    assert example_rst == want_rst
 
 
 def test_rst_empty_code_block(gallery_conf, tmpdir):
@@ -191,15 +201,20 @@ def test_rst_empty_code_block(gallery_conf, tmpdir):
         blocks, script_vars, gallery_conf)
 
     example_rst = sg.rst_blocks(blocks, output_blocks, file_conf, gallery_conf)
-    assert example_rst.rstrip('\n') == """Docstring
+    want_rst = """\
+Docstring
+
+.. GENERATED FROM PYTHON SOURCE LINES 3-4
 
 Paragraph 1
 
+.. GENERATED FROM PYTHON SOURCE LINES 4-5
 
 .. code-block:: python
 
 
     # just a comment"""
+    assert example_rst.rstrip('\n') == want_rst
 
 
 def test_script_vars_globals(gallery_conf, tmpdir):
@@ -235,7 +250,7 @@ b = 'foo'
 def test_codestr2rst():
     """Test the correct translation of a code block into rst."""
     output = sg.codestr2rst('print("hello world")')
-    reference = """
+    reference = """\
 .. code-block:: python
 
     print("hello world")"""
@@ -431,10 +446,24 @@ def test_remove_config_comments(gallery_conf, req_pil):
     """Test the gallery_conf['remove_config_comments'] setting."""
     rst = _generate_rst(gallery_conf, 'test.py', CONTENT)
     assert '# sphinx_gallery_thumbnail_number = 1' in rst
+    assert '# sphinx_gallery_defer_figures' in rst
 
     gallery_conf['remove_config_comments'] = True
     rst = _generate_rst(gallery_conf, 'test.py', CONTENT)
     assert '# sphinx_gallery_thumbnail_number = 1' not in rst
+    assert '# sphinx_gallery_defer_figures' not in rst
+
+
+def test_final_empty_block(gallery_conf, req_pil):
+    """Test empty final block is removed. Empty final block can occur after
+    sole config comment is removed from final block."""
+    CONTENT.extend(
+        ('# %%', '', '# sphinx_gallery_line_numbers = True')
+    )
+    gallery_conf['remove_config_comments'] = True
+    rst = _generate_rst(gallery_conf, 'test.py', CONTENT)
+    want = "RuntimeWarning)\n\n\n.. rst-class:: sphx-glr-timing"
+    assert want in rst
 
 
 def test_download_link_note_only_html(gallery_conf, req_pil):
@@ -536,14 +565,17 @@ def test_thumbnail_path(test_str):
 
 def test_zip_notebooks(gallery_conf):
     """Test generated zipfiles are not corrupt."""
-    gallery_conf.update(examples_dir='examples')
+    gallery_conf.update(
+        examples_dir=os.path.join(gallery_conf['src_dir'], 'examples'))
+    shutil.copytree(
+        os.path.join(os.path.dirname(__file__), 'tinybuild', 'examples'),
+        gallery_conf['examples_dir'])
     examples = downloads.list_downloadable_sources(
         gallery_conf['examples_dir'])
     zipfilepath = downloads.python_zip(examples, gallery_conf['gallery_dir'])
     zipf = zipfile.ZipFile(zipfilepath)
     check = zipf.testzip()
-    if check:
-        raise OSError("Bad file in zipfile: {0}".format(check))
+    assert not check, "Bad file in zipfile: {0}".format(check)
 
 
 def test_rst_example(gallery_conf):
@@ -555,7 +587,8 @@ def test_rst_example(gallery_conf):
                                 'dependencies': './binder/requirements.txt',
                                 'notebooks_dir': 'notebooks',
                                 'use_jupyter_lab': True,
-                                })
+                                },
+                        gallery_dirs=None)
 
     example_file = os.path.join(gallery_conf['gallery_dir'], "plot.py")
     sg.save_rst_example("example_rst", example_file, 0, 0, gallery_conf)
@@ -674,7 +707,9 @@ fig
 
 html_out = """.. raw:: html
 
+    <div class="output_subarea output_html rendered_html output_result">
     <div> This is the _repr_html_ div </div>
+    </div>
     <br />
     <br />"""
 
@@ -710,7 +745,7 @@ def _clean_output(output):
 @pytest.mark.parametrize('capture_repr, code, expected_out', [
     pytest.param(tuple(), 'a=2\nb=3', '', id='assign,()'),
     pytest.param(tuple(), 'a=2\na', '', id='var,()'),
-    pytest.param(tuple(), 'a=2\nprint(a)', '2',  id='print(var),()'),
+    pytest.param(tuple(), 'a=2\nprint(a)', '2', id='print(var),()'),
     pytest.param(tuple(), 'print("hello")\na=2\na', 'hello',
                  id='print+var,()'),
     pytest.param(('__repr__',), 'a=2\nb=3', '', id='assign,(repr)'),
