@@ -1,8 +1,8 @@
 import os
 
 import pytest
-import numpy as np
 
+from sphinx.errors import ConfigError, ExtensionError
 import sphinx_gallery
 from sphinx_gallery.gen_gallery import _complete_gallery_conf
 from sphinx_gallery.scrapers import (figure_rst, mayavi_scraper, SINGLE_IMAGE,
@@ -14,6 +14,9 @@ from sphinx_gallery.utils import _get_image
 @pytest.fixture(scope='function')
 def gallery_conf(tmpdir):
     """Sets up a test sphinx-gallery configuration"""
+    # Skip if numpy not installed
+    pytest.importorskip("numpy")
+
     gallery_conf = _complete_gallery_conf({}, str(tmpdir), True, False)
     gallery_conf.update(examples_dir=str(tmpdir), gallery_dir=str(tmpdir))
     return gallery_conf
@@ -63,6 +66,7 @@ def test_save_matplotlib_figures(gallery_conf, ext, req_mpl, req_pil):
 
 def test_save_mayavi_figures(gallery_conf, req_mpl, req_pil):
     """Test file naming when saving figures. Requires mayavi."""
+    import numpy as np
     Image = _get_image()
     try:
         from mayavi import mlab
@@ -130,42 +134,43 @@ def test_custom_scraper(gallery_conf, monkeypatch):
                   lambda: _custom_func, raising=False)
         for cust in (_custom_func, 'sphinx_gallery'):
             gallery_conf.update(image_scrapers=[cust])
-            _complete_gallery_conf(*complete_args)  # smoke test that it works
+            # smoke test that it works
+            _complete_gallery_conf(*complete_args, check_keys=False)
     # degenerate
     # without the monkey patch to add sphinx_gallery._get_sg_image_scraper,
     # we should get an error
     gallery_conf.update(image_scrapers=['sphinx_gallery'])
-    with pytest.raises(ValueError,
+    with pytest.raises(ConfigError,
                        match="has no attribute '_get_sg_image_scraper'"):
-        _complete_gallery_conf(*complete_args)
+        _complete_gallery_conf(*complete_args, check_keys=False)
 
     # other degenerate conditions
     gallery_conf.update(image_scrapers=['foo'])
-    with pytest.raises(ValueError, match='Unknown image scraper'):
-        _complete_gallery_conf(*complete_args)
+    with pytest.raises(ConfigError, match='Unknown image scraper'):
+        _complete_gallery_conf(*complete_args, check_keys=False)
     gallery_conf.update(image_scrapers=[_custom_func])
     fname_template = os.path.join(gallery_conf['gallery_dir'],
                                   'image{0}.png')
     image_path_iterator = ImagePathIterator(fname_template)
     block = ('',) * 3
     block_vars = dict(image_path_iterator=image_path_iterator)
-    with pytest.raises(RuntimeError, match='did not produce expected image'):
+    with pytest.raises(ExtensionError, match='did not produce expected image'):
         save_figures(block, block_vars, gallery_conf)
     gallery_conf.update(image_scrapers=[lambda x, y, z: 1.])
-    with pytest.raises(TypeError, match='was not a string'):
+    with pytest.raises(ExtensionError, match='was not a string'):
         save_figures(block, block_vars, gallery_conf)
     # degenerate string interface
     gallery_conf.update(image_scrapers=['sphinx_gallery'])
     with monkeypatch.context() as m:
         m.setattr(sphinx_gallery, '_get_sg_image_scraper', 'foo',
                   raising=False)
-        with pytest.raises(ValueError, match='^Unknown image.*\n.*callable'):
-            _complete_gallery_conf(*complete_args)
+        with pytest.raises(ConfigError, match='^Unknown image.*\n.*callable'):
+            _complete_gallery_conf(*complete_args, check_keys=False)
     with monkeypatch.context() as m:
         m.setattr(sphinx_gallery, '_get_sg_image_scraper', lambda: 'foo',
                   raising=False)
-        with pytest.raises(ValueError, match='^Scraper.*was not callable'):
-            _complete_gallery_conf(*complete_args)
+        with pytest.raises(ConfigError, match='^Scraper.*was not callable'):
+            _complete_gallery_conf(*complete_args, check_keys=False)
 
 
 @pytest.mark.parametrize('ext', _KNOWN_IMG_EXTS)
@@ -175,6 +180,7 @@ def test_figure_rst(ext):
     image_rst = figure_rst(figure_list, '.')
     single_image = """
 .. image:: /sphx_glr_plot_1.{ext}
+    :alt: pl
     :class: sphx-glr-single-img
 """.format(ext=ext)
     assert image_rst == single_image
@@ -188,12 +194,14 @@ def test_figure_rst(ext):
     *
 
       .. image:: /sphx_glr_plot_1.{ext}
-            :class: sphx-glr-multi-img
+          :alt: pl
+          :class: sphx-glr-multi-img
 
     *
 
       .. image:: /second.{ext}
-            :class: sphx-glr-multi-img
+          :alt: pl
+          :class: sphx-glr-multi-img
 """.format(ext=ext)
     assert image_rst == image_list_rst
 
@@ -201,7 +209,7 @@ def test_figure_rst(ext):
     local_img = [os.path.join(os.getcwd(), 'third.' + ext)]
     image_rst = figure_rst(local_img, '.')
 
-    single_image = SINGLE_IMAGE % ("third." + ext)
+    single_image = SINGLE_IMAGE % ("third." + ext, '')
     assert image_rst == single_image
 
 
@@ -209,6 +217,6 @@ def test_iterator():
     """Test ImagePathIterator."""
     ipi = ImagePathIterator('foo{0}')
     ipi._stop = 10
-    with pytest.raises(RuntimeError, match='10 images'):
+    with pytest.raises(ExtensionError, match='10 images'):
         for ii in ipi:
             pass
